const lo = require('lodash')
const hlp = require('../../helpers')
const hlpPolicy = require('../../helpers_policy')
const consts = require('../../consts')
const settConverters = require('./sections/index.js')
const settCommonData = require('./consts.js')

function Settings () {
    let changesList = []
    let settingsCache = {}
    let policiesCache = {}
    let rbacCache = {}
    let curHostId = ''
    let activePolicyId = -1

    function AddToChangesList (arrSections) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
        for (let section of arrSections) {
            if (section!=='' && !changesList.includes(section)) {
                changesList.push(section)
            }
        }
    }
    /*
     * Метод добавляет в cache политик новые секции
     */
    async function UpdateSettingsCache (api, arrSections, hostId) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
        for (let section of arrSections) {
            if( consts.policySsSections[section]=== undefined /*|| settingsCache[section]!==undefined*/)
                continue
            let sectionData = {}
            try{
              sectionData = await api.HostGroup.SS_Read({
                                strHostName: hostId,
                                strType: 'SS_SETTINGS',
                                strProduct: consts.wstrProduct,
                                strVersion: consts.wstrVersion,
                                strSection: consts.policySsSections[section] })
            }
            catch (error) {
                console.error(error)
            }
            settingsCache[section] = (sectionData.result!== undefined && sectionData.result.PxgError===undefined) ? sectionData.result : settCommonData.Defaults[section]
        }
    }

    async function GetRbacCache (api, section) {
        if( consts.policySsSections[section]===undefined )
            return {}

        try{
            let ReadonlyNodes = {}
            await api.HstAccessControl.GetSettingsReadonlyNodes({
                lGroupId: curHostId,
                szwProduct: consts.wstrProduct,
                szwProduct: consts.wstrVersion,
                szwSectionName: consts.policySsSections[section],
                pSettingsSection:ReadonlyNodes
            })
            return ReadonlyNodes;
        }
        catch (error) {
            console.error(error)
        }
        return {};
    }
    /*
     * Called via api.js in getFormModel
     */
    async function getSettingsData (api, section, hostId) {
        if(curHostId != hostId) {
            curHostId = hostId
            changesList = []
            settingsCache = {}
            policiesCache = {}
        }
        activePolicyId = await hlpPolicy.GetHostActivePolicyId(api, hostId);

        if (section === undefined || settConverters[section] === undefined)
            return {}

        if (settConverters[section].sections !== undefined) {
            const settingsSSSection = settConverters[section].sections
            await UpdateSettingsCache(api, settingsSSSection, hostId);
            if(activePolicyId===-1){
                hlpPolicy.UpdatePoliciesCacheAsDefault(settingsSSSection, policiesCache, settCommonData.PolicyMappper);
            } else {
                await hlpPolicy.UpdatePoliciesCache(api, activePolicyId, settingsSSSection, policiesCache, settCommonData.PolicyMappper);
            }
//            await UpdatePoliciesCache(api, settConverters[section].sections);
        }

        return (settConverters[section].toView !== undefined)  ? settConverters[section].toView(settingsCache,policiesCache) : {}
    }
    /*
     * Called via api.js in setFormModel
     */
    async function updateSettingsData (kscApi, section, data) {
        if (section === undefined || settConverters[section] === undefined)
            return
        if (settConverters[section].sections !== undefined) {
            AddToChangesList(settConverters[section].sections)
        }
        if(settConverters[section].toRawEx!==undefined)
            await settConverters[section].toRawEx(kscApi, data, settingsCache)
        else
            settConverters[section].toRaw(data, settingsCache)
    }
    /*
     * Called via api.js in beforeApplyChanges
     */
    function commitSettingsData (api) {
    }
    /*
     * Called via api.js in applyChanges
     */
    async function saveSettingsData (api, hostId) {
        if(changesList.length === 0 )
            return
        if(!hostId)
            hostId = curHostId

        for (let section of changesList) {
            try{
                await api.HostGroup.SS_Write({
                    strHostName: hostId,
                    strType: 'SS_SETTINGS',
                    strProduct: consts.wstrProduct,
                    strVersion: consts.wstrVersion,
                    strSection: consts.policySsSections[section],
                    nOption: 3,
                    pSettings: settingsCache[section]
                })
            }
            catch (error) {
                console.error(error)
            }
        }

        changesList = []
        settingsCache = {}
    }

    return {
        getSettingsData,
        updateSettingsData,
        commitSettingsData,
        saveSettingsData
    }
}

module.exports = Settings
