const lo = require('lodash')
//const util = require('util')
const { helpers } = require('@kl/openapi-client')
const taskConverters = {
	"remove-file": require('./types/remove-file.js'),
	"quarantine-file": require('./types/quarantine-file.js'),
	"restore-file": require('./types/restore-file.js'),
	"ioc-scan": require('./types/ioc-scan.js'),
	"terminate-process": require('./types/terminate-process.js'),
	"execute-command": require('./types/execute-command.js')
}
const TaskTypeSettingsValue = 'PluginTaskType'

function taskInfo() {
    let taskAdditionalParams = { } //initial data
    let changesToSave = []
    let currentTaskType = ''
    let curTaskId = ''
	
    async function mapTaskData ({ kscApi, taskData, formType }) {
        console.log('ksws_windows:> taskInfo::mapTaskData ' + JSON.stringify(arguments))
        console.log('ksws_windows:> taskInfo::mapTaskData, currentTaskType ' + currentTaskType)
        let computedData = {}
        let {TASK_ADDITIONAL_PARAMS} = taskData
        let converter = taskConverters[currentTaskType]
        if (converter) {
            computedData = converter.toView(TASK_ADDITIONAL_PARAMS, formType)
        }
        computedData[TaskTypeSettingsValue] = currentTaskType
        taskData.computedData = computedData
        return taskData
    }

    function getTaskAdditionalParams()
    {
        console.log('ksws_windows:> taskInfo::getTaskAdditionalParams taskAdditionalParams:' + JSON.stringify(taskAdditionalParams))
        return taskAdditionalParams
    }

    async function getTaskData ({kscApi, taskId, hostId, formType}) {
        console.log('ksws_windows:> getTaskData ' + JSON.stringify(arguments))
        let result
        let id = taskId.toString()

        if (hostId){
            let strHostName = hostId.toString()
            let {result: strSrvObjId} = await kscApi.HostGroup.GetHostTasks({strHostName})
            let task = await kscApi.HostTasks.GetTaskData({strTask: id, strSrvObjId})
            result = task.result
        }
        else {
            let task = await kscApi.Tasks.GetTaskData({taskId: id, skipDeserialization: true})
            result = helpers.serializer.deserializeKscRequest({body: task.result}).result
        }

        if (!result || !result.TASK_ADDITIONAL_PARAMS) {
            return {}
        }

        taskAdditionalParams = lo.clone(result.TASK_ADDITIONAL_PARAMS)
        currentTaskType = result.TASK_NAME
        let outData = await mapTaskData({kscApi, taskData: result, formType})
        return outData.computedData
    }

    async function updateTaskData ({kscApi, modelData, formType}) {
        console.log('ksws_windows:> updateTaskData ' + JSON.stringify(arguments))
        console.log('ksws_windows:> updateTaskData, currentTaskType ' + currentTaskType)
        let converter = taskConverters[currentTaskType]
        if (converter) {
            converter.toRaw(taskAdditionalParams, modelData, formType)
            console.log('ksws_windows:> updateTaskData Converter taskAdditionalParams:' + JSON.stringify(taskAdditionalParams))
        }
    }

    async function getWizardTaskData ({kscApi, taskType, formType, defData}) {
        console.log('ksws_windows:> getWizardTaskData ' + JSON.stringify(arguments))

        let Data

        let tmpData = {}
        tmpData.PxgRetVal = lo.clone(defData)

        Data = helpers.serializer.deserializeKscRequest({body: tmpData}).result

        if (!Data || !Data.TASK_ADDITIONAL_PARAMS) {
            return {}
        }

        taskAdditionalParams = lo.clone(Data.TASK_ADDITIONAL_PARAMS)
        currentTaskType = taskType

        let outData = await mapTaskData({kscApi, taskData: Data, formType})
        return outData.computedData
    }

    return {
        getTaskData,
        getTaskAdditionalParams,
        updateTaskData,
        getWizardTaskData
    }

}

module.exports = taskInfo
